@@ -39,6 +39,7 @@
 #include <Lum/ButtonRow.h>
 #include <Lum/Dialog.h>
 #include <Lum/Panel.h>
+#include <Lum/Toggle.h>
 
 #include "Configuration.h"
 #include "Selector.h"
@@ -53,35 +54,37 @@ class MainDialog : public Lum::Dialog
 private:
   enum State {
     stateUninitialized,
-    stateStoped,
     stateRunning,
+    statePaused,
+    stateStoped,
     stateAlerting
   };
 
 private:
-  Lum::Model::ActionRef aboutAction;
-  Lum::Model::ActionRef startAction;
-  Lum::Model::ActionRef restartAction;
-  Lum::Model::ActionRef stopAction;
-  Lum::Model::ActionRef timer;
-  Lum::Model::SizeTRef  seconds;
-  size_t                lastSeconds;
+  Lum::Model::ActionRef  aboutAction;
+  Lum::Model::ActionRef  startAction;
+  Lum::Model::BooleanRef paused;
+  Lum::Model::ActionRef  stopAction;
+  Lum::Model::ActionRef  timer;
+  Lum::Model::SizeTRef   seconds;
 
-  State                 state;
-  Lum::Base::SystemTime alertTime;
+  size_t                 initialSeconds;
+  Lum::Base::Timer       time;
 
-  StopWatch             *value;
-  Lum::Audio::PlaySound *beepSound;
+  State                  state;
+  StopWatch              *value;
+  Lum::Base::SystemTime  alertTime;
+  Lum::Audio::PlaySound  *beepSound;
 
 public:
   MainDialog()
   : aboutAction(new Lum::Model::Action()),
     startAction(new Lum::Model::Action()),
-    restartAction(new Lum::Model::Action()),
+    paused(new Lum::Model::Boolean(false)),
     stopAction(new Lum::Model::Action()),
     timer(new Lum::Model::Action()),
     seconds(new Lum::Model::SizeT()),
-    lastSeconds(0),
+    initialSeconds(0),
     state(stateUninitialized),
     value(new StopWatch()),
     beepSound(NULL)
@@ -117,14 +120,14 @@ public:
 
     Observe(aboutAction);
     Observe(startAction);
-    Observe(restartAction);
+    Observe(paused);
     Observe(stopAction);
     Observe(timer);
 
     // Settings for initial state
     value->GetClickAction()->Enable();
     startAction->Disable();
-    restartAction->Disable();
+    paused->Disable();
     stopAction->Disable();
   }
 
@@ -139,16 +142,7 @@ public:
       value->SetText(L"-:--");
     }
     else {
-      std::wstring text;
-
-      text=Lum::Base::NumberToWString(seconds->Get()/60);
-      text.append(L":");
-      if (seconds->Get()%60<10) {
-        text.append(L"0");
-      }
-      text.append(Lum::Base::NumberToWString(seconds->Get()%60));
-
-      value->SetText(text);
+      value->SetText(Lum::Base::TimeSpanToWString(seconds->Get()));
     }
   }
 
@@ -179,52 +173,81 @@ public:
 
     this->state=state;
 
-    // Right
+    // Visual representation
 
     switch (state) {
     case stateUninitialized:
       assert(false);
       break;
-    case stateStoped:
+    case stateRunning:
+      value->GetClickAction()->Disable();
+      value->SetAlarm(false);
       startAction->Enable();
-      restartAction->Enable();
-      stopAction->Disable();
+      paused->Enable();
+      stopAction->Enable();
+      AllowScreenBlanking(false);
+    case statePaused:
+      value->GetClickAction()->Disable();
+      value->SetAlarm(false);
+      startAction->Enable();
+      paused->Enable();
+      stopAction->Enable();
+      AllowScreenBlanking(false);
+      break;
+    case stateStoped:
       value->GetClickAction()->Enable();
+      value->SetAlarm(false);
+      startAction->Enable();
+      paused->Disable();
+      stopAction->Disable();
       AllowScreenBlanking(true);
       break;
-    case stateRunning:
-      restartAction->Enable();
-      value->GetClickAction()->Disable();
-      AllowScreenBlanking(false);
     case stateAlerting:
-      startAction->Enable();
-      restartAction->Enable();
-      stopAction->Enable();
       value->GetClickAction()->Disable();
+      value->SetAlarm(true);
+      startAction->Enable();
+      paused->Disable();
+      stopAction->Enable();
       AllowScreenBlanking(false);
       break;
     }
 
-    // Action
+    // Timer & state
 
     switch (state) {
     case stateUninitialized:
       assert(false);
       break;
-    case stateStoped:
-      value->SetAlarm(false);
-      Lum::OS::display->RemoveTimer(timer);
-      break;
     case stateRunning:
-      value->SetAlarm(false);
-      lastSeconds=seconds->Get();
+      paused->Set(false);
+      if (time.IsPausing()) {
+        time.Resume();
+      }
+      else {
+        time.Start();
+      }
+      seconds->Set(initialSeconds-time.GetTime());
       alertTime.SetToNow();
       alertTime.Add(seconds->Get());
       Lum::OS::display->AddTimer(0,100000,timer);
       break;
+    case statePaused:
+      seconds->Set(initialSeconds-time.GetTime());
+      time.Pause();
+      Lum::OS::display->RemoveTimer(timer);
+      break;
+    case stateStoped:
+      if (time.IsStarted()) {
+        seconds->Set(initialSeconds-time.GetTime());
+      }
+      time.Reset();
+      paused->Set(false);
+      Lum::OS::display->RemoveTimer(timer);
+      break;
     case stateAlerting:
-      value->SetAlarm(true);
-      seconds->Set(0);
+      seconds->Set(initialSeconds-time.GetTime());
+      time.Reset();
+      paused->Set(false);
       PlayAlert();
       alertTime.SetToNow();
       alertTime.Add(3);
@@ -242,8 +265,9 @@ public:
             ->Add(value)
             ->AddSpace()
             ->Add(Lum::ButtonRow::Create(true,false)
+                  ->Add(Lum::Button::Create(_(L"BUTTON_SETTINGS",L"S_ettings..."),value->GetClickAction(),true,true))
                   ->Add(Lum::Button::Create(_(L"BUTTON_START",L"_Start"),startAction,true,true))
-                  ->Add(Lum::Button::Create(_(L"BUTTON_RESTART",L"_Restart"),restartAction,true,true))
+                  ->Add(Lum::Toggle::Create(_(L"BUTTON_PAUSE",L"Pause"),paused,true,true))
                   ->Add(Lum::Button::Create(_(L"BUTTON_STOP",L"Sto_p"),stopAction,true,true))));
 
     Lum::Def::Menu *menu=Lum::Def::Menu::Create();
@@ -294,19 +318,27 @@ public:
         selector->Close();
       }
 
+      if (selector->Success()) {
+        initialSeconds=seconds->Get();
+        SetState(stateStoped);
+      }
       delete selector;
 
-      SetState(stateStoped);
     }
     else if (model==startAction &&
              startAction->IsFinished()) {
+      SetState(stateStoped);
       SetState(stateRunning);
     }
-    else if (model==restartAction &&
-             restartAction->IsFinished()) {
-      SetState(stateStoped);
-      seconds->Set(lastSeconds);
-      SetState(stateRunning);
+    else if (model==paused) {
+      if (state==stateRunning &&
+          paused->Get()) {
+        SetState(statePaused);
+      }
+      else if (state==statePaused &&
+               !paused->Get()) {
+        SetState(stateRunning);
+      }
     }
     else if (model==stopAction &&
              stopAction->IsFinished()) {
@@ -321,7 +353,7 @@ public:
           SetState(stateAlerting);
         }
         else {
-          seconds->Set((alertTime.GetTime()-now.GetTime()));
+          seconds->Set(initialSeconds-time.GetTime());
           Lum::OS::display->AddTimer(0,100000,timer);
         }
       }
