@@ -21,7 +21,8 @@
 
 #include <Lum/Base/String.h>
 
-//static Lum::OS::Display::FontType fontType  = Lum::OS::Display::fontTypeProportional;
+#include "config.h"
+
 static Lum::OS::Display::FontType fontType  = Lum::OS::Display::fontTypeFixed;
 static unsigned long              fontStyle = Lum::OS::Font::normal;
 
@@ -90,6 +91,12 @@ Lum::OS::Font* StopWatch::FindFont(size_t& digitWidth, size_t& divWidth)
 {
   size_t fontSize=height;
 
+#if defined(HAVE_LIB_HILDON)
+  if (height==338) {
+    fontSize=271;
+  }
+#endif
+
   while (true) {
     Lum::OS::FontRef tmpFont;
     size_t           w=0;
@@ -105,7 +112,7 @@ Lum::OS::Font* StopWatch::FindFont(size_t& digitWidth, size_t& divWidth)
       digitWidth=w;
       divWidth=tmpFont->StringWidth(L":",fontStyle);
 
-      if (3*digitWidth+divWidth+2*digitWidth<=width && tmpFont->height<=height) {
+      if (3*digitWidth+divWidth+2*digitWidth<=width && tmpFont->pixelHeight<=height) {
         return tmpFont;
       }
     }
